-- Some helping functions
function ceil(value, place)
    place = place or 0
    local factor = 10^place
	return (math.ceil(value / factor) * factor)
end

function floor(value, place)
    place = place or 0
    local factor = 10^place
	return (math.floor(value / factor) * factor)
end

function round(number, place)
    place = place or 0
    local factor = 10^place
    if number >= 0 then
        return math.floor(number / factor + 0.5) * factor
    else
        return math.ceil(number / factor - 0.5) * factor
    end
end

function bonus_cap(bonus) -- prevents int overflow
    local sign = round(math.abs(bonus)/bonus)
    if math.abs(bonus) > 327.67 then
        return 327.67 * sign
    else
        return bonus
    end
end

function find_index(table, input_string)
    for index, value in pairs(table) do
        if value == input_string then
            return index
        end
    end
    return nil  -- Return nil if the string is not found
end

function manual_override_pattern(input)
    local result = {}
    while true do
        local pos = string.find(input, ",") -- Find the position of the next comma
        local substring = pos and string.sub(input, 1, pos - 1) or input -- Extract the substring before the comma
        input = pos and string.sub(input, pos + 1) or "" -- Remove the processed substring from the input
        if tonumber(substring) then
            table.insert(result, tonumber(substring))
        else
            table.insert(result, "nil")
        end
        if input == "" then
            break
        end
    end
    return result
end

function add_numbers_to_icons(module, prototype, range_min, range_max)  -- Graphics
    local secondary_tint = module.graphics.secondary_tint
    for level = range_min, range_max do
        local hundreds = floor(level,2)/100
        local tens = floor((level - hundreds*100),1)/10
        local ones = level - hundreds*100 - tens*10

        if hundreds > 0 then
            data.raw[prototype][module.name.."-module-"..level]["icons"][2].icon = "__module-config__/graphics/frame.png"
            data.raw[prototype][module.name.."-module-"..level]["icons"][3].icon = "__module-config__/graphics/Hundreds/"..hundreds..".png"
            data.raw[prototype][module.name.."-module-"..level]["icons"][4].icon = "__module-config__/graphics/Tens/"..tens..".png"
            data.raw[prototype][module.name.."-module-"..level]["icons"][5].icon = "__module-config__/graphics/Ones/"..ones..".png"
        elseif tens > 0 then
            data.raw[prototype][module.name.."-module-"..level]["icons"][2].icon = "__module-config__/graphics/xframex.png"
            data.raw[prototype][module.name.."-module-"..level]["icons"][4].icon = "__module-config__/graphics/Tens/"..tens..".png"
            data.raw[prototype][module.name.."-module-"..level]["icons"][5].icon = "__module-config__/graphics/Ones/"..ones..".png"
        else
            data.raw[prototype][module.name.."-module-"..level]["icons"][5].icon = "__module-config__/graphics/Ones/"..ones..".png"
        end
        data.raw[prototype][module.name.."-module-"..level]["icons"][3].tint = {secondary_tint[1], secondary_tint[2], secondary_tint[3]}
        data.raw[prototype][module.name.."-module-"..level]["icons"][4].tint = {secondary_tint[1], secondary_tint[2], secondary_tint[3]}
        data.raw[prototype][module.name.."-module-"..level]["icons"][5].tint = {secondary_tint[1], secondary_tint[2], secondary_tint[3]}
    end
end

function tech_cost_pattern(expression, max_level)
    local cost = {}
    
    -- Precompile the formula, replacing "L" with the variable `level`
    local formula = "return function(L) return " .. expression .. " end"
    local compiled_formula, err = load(formula)
    
    if not compiled_formula then
        -- If there's an error in compiling the formula, fill the cost table with 1s
        for level = 1, max_level do
            cost[level] = 1
        end
        return cost
    end

    compiled_formula = compiled_formula()
    
    -- Evaluate the formula for each level, using pcall to catch runtime errors
    for level = 1, max_level do
        local success, result = pcall(compiled_formula, level)
        if success then
            if result == 0 then
                result = 1
            end
            cost[level] = ceil(math.abs(result))
        else
            -- If there's an error in evaluating the formula, set the cost to 1
            cost[level] = 1
        end
    end
    
    return cost
end

-- Helper function to get the bonus
function get_bonus(scaling_type, initial_value, max_level, scale, offset, effect, extra)
    -- Defining what bonuses modules should have
    -- Define the pattern functions
    local pattern_functions = {
        ["disabled_pattern"] = disabled_pattern,
        ["linear_pattern"] = linear_pattern,
        ["constant_pattern"] = constant_pattern,
        ["fibonacci_pattern"] = fibonacci_pattern,
        ["exponential_pattern"] = exponential_pattern,
        ["polynomial_pattern"] = polynomial_pattern,
        ["linear_pattern-advanced"] = linear_pattern_advanced,
        ["constant_pattern-advanced"] = constant_pattern_advanced,
        ["fibonacci_pattern-advanced"] = fibonacci_pattern_advanced,
        ["changing_scale_pattern-advanced"] = changing_scale_pattern_advanced,
        ["exponential_pattern-advanced"] = exponential_pattern_advanced,
        ["polynomial_pattern-advanced"] = polynomial_pattern_advanced,
    }
    if string.find(scaling_type, "-advanced") then
        return pattern_functions[scaling_type](initial_value, max_level, scale, offset, extra)
    else
        return pattern_functions[scaling_type](max_level, effect)
    end
end

function generate_effects(modules, module_type)
    local effect_types = {"consumption", "speed", "productivity", "pollution"}
    if mods["quality"] then
        table.insert(effect_types, "quality")
    end
    local effect = {}

    for _, effect_type in pairs(effect_types) do
        effect[effect_type] = get_bonus(
            modules[module_type].scaling[effect_type],
            settings.startup[module_type.."-"..effect_type.."-initial-value"].value,
            modules[module_type].max_level,
            settings.startup[module_type.."-"..effect_type.."-scale"].value,
            settings.startup[module_type.."-"..effect_type.."-offset"].value,
            {data.raw["module"][module_type.."-module"].effect[effect_type], data.raw["module"][module_type.."-module-2"].effect[effect_type], data.raw["module"][module_type.."-module-3"].effect[effect_type]},
            settings.startup[module_type.."-"..effect_type.."-extra"].value
        )
    end
    return effect
end

function generate_module(modules, module_type, level, require_alt_mode)
    data:extend({
        {
            type = "module",
            name = module_type.."-module-"..level,
            localised_name = modules[module_type].local_name..level,
            localised_description = modules[module_type].local_description,
            icons = {
                {
                    icon = "__module-config__/graphics/"..module_type.."-module-3.png",
                    icon_size = 64
                },
                {
                    icon = "__module-config__/graphics/xxframe.png",
                    icon_size = 64
                },
                {
                    icon = "__module-config__/graphics/empty.png", -- Hundreds place
                    icon_size = 64,
                    tint = {r = 1, g = 1, b = 1, a = 1}
                },
                {
                    icon = "__module-config__/graphics/empty.png", -- Tens place
                    icon_size = 64,
                    tint = {r = 1, g = 1, b = 1, a = 1}
                },
                {
                    icon = "__module-config__/graphics/empty.png", -- Ones place
                    icon_size = 64,
                    tint = {r = 1, g = 1, b = 1, a = 1}
                }
            },
            subgroup = modules[module_type].subgroup,
            category = module_type,
            tier = level,
            order = modules[module_type].order.."["..module_type.."]-a["..module_type.."-module-"..level.."]",
            stack_size = 50,
            effect = {
                consumption = modules[module_type]["effect"]["consumption"][level],
                speed = modules[module_type]["effect"]["speed"][level],
                productivity = modules[module_type]["effect"]["productivity"][level],
                pollution = modules[module_type]["effect"]["pollution"][level]
            },
            art_style = "vanilla",
            requires_beacon_alt_mode = require_alt_mode,
            beacon_tint = {
                primary = modules[module_type].graphics.primary_tint,
                secondary = modules[module_type].graphics.secondary_tint,
            }
        }
    })
    if mods["quality"] then
        data.raw["module"][module_type.."-module-"..level]["effect"].quality = modules[module_type]["effect"]["quality"][level]
    end
    if level < 100 then -- updating order
        if level < 10 then
            data.raw["module"][module_type.."-module-"..level]["order"] = modules[module_type].order.."["..module_type.."]-a["..module_type.."-module-00"..level.."]"
        else
            data.raw["module"][module_type.."-module-"..level]["order"] = modules[module_type].order.."["..module_type.."]-a["..module_type.."-module-0"..level.."]"
        end
    end
end



--------------------- Some functions which determine how effects scale ---------------------

--------------------- BASIC ---------------------
---Basic Functions scale entirely from existing effects
-- Bonuses scale via fibonacci pattern    BASIC MODE
function fibonacci_pattern(length, initial_numbers)
    local effects = {initial_numbers[1], initial_numbers[2], initial_numbers[3]}
    local initial_length = #initial_numbers

    -- Generate the Fibonacci sequence past the starting levels
    for i = initial_length, length do
        effects[i] = effects[i - 1] + effects[i - 2]
        effects[i] = bonus_cap(round(effects[i], -4))
    end

    return effects
end

-- Bonuses scale via linear pattern    BASIC MODE
function linear_pattern(length, initial_numbers)
    local effects = {initial_numbers[1], initial_numbers[2], initial_numbers[3]}
    local initial_length = #initial_numbers
    
    -- Finding the scaling pattern (i*scale = bonus)
    local scale = round((initial_numbers[3] - initial_numbers[2]) - (initial_numbers[2] - initial_numbers[1]),-2)
    if scale == 0 then
        scale = round((initial_numbers[2] - initial_numbers[1]),-2)
    end
    local offset = round(((initial_numbers[2] - initial_numbers[1]) / scale - 1))
    if (offset < 0) or (offset == math.huge) or (offset ~= offset) then
        offset = 0
    end
    -- Generate the effects knowing the scale factor
    for i = initial_length, length do
        effects[i] = effects[i-1] + (i-1 + offset) * scale
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end


-- Bonuses scale by a constant value    BASIC MODE
function constant_pattern(length, initial_numbers)
    local effects = {initial_numbers[1], initial_numbers[2], initial_numbers[3]}
    local initial_length = #initial_numbers

    local scale = initial_numbers[3] - initial_numbers[2]

    -- Generate the effects knowing the scale factor
    for i = initial_length, length do
        effects[i] = effects[i-1] + scale
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end


function exponential_pattern(length, initial_numbers)
    local effects = {initial_numbers[1], initial_numbers[2], initial_numbers[3]}

    local scale = 2

    -- Generate the effects knowing the scale factor
    for i = 2, length do
        effects[i] = effects[1] * scale^(i-1)
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end


function polynomial_pattern(length, initial_numbers)
    local effects = {initial_numbers[1], initial_numbers[2], initial_numbers[3]}

    local scale = 2

    -- Generate the effects knowing the scale factor
    for i = 2, length do
        effects[i] = effects[1] * i^scale
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end

function disabled_pattern(length, initial_numbers)
    local effects = {0}
    for i = 1, length do
        effects[i] = 0
    end
    return effects
end


--------------------- ADVANCED ---------------------

function fibonacci_pattern_advanced(start, length, scale, offset, extra) -- Extra will determine how often to increase effects NEED TO ADD HERE
    local effects = {}
    offset = round(offset)
    effects[1] = start
    effects[2] = scale + start

    for i = 3, length + offset do
        effects[i] = effects[i-1] + effects[i-2]
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    while offset > 0 do
        table.remove(effects, 1)
        offset = offset - 1
    end
    return effects
end


function linear_pattern_advanced(start, length, scale, offset, extra) -- Extra will determine how often to increase effects
    extra = round(extra)
    offset = round(offset)
    if extra == 0 then
        extra = 1
    end
    local effects = {}
    local tier = 2
    effects[1] = start
    for i = 2, length do
        if ((i-1) % extra == 0) then
            effects[i] = scale*(tier+offset-1) + effects[i-1]
            effects[i] = bonus_cap(round(effects[i], -4))
            tier = tier + 1
        else
            effects[i] = effects[i-1]
        end
    end
    return effects
end


function constant_pattern_advanced(start, length, scale, offset, extra) -- Extra will determine how often to increase effects.
    extra = round(extra) 
    if extra == 0 then
        extra = 1
    end
    local effects = {}
    effects[1] = start
    for i = 2, length do
        if ((i-1) % extra == 0) then
            effects[i] = scale + effects[i-1]
            effects[i] = bonus_cap(round(effects[i], -4))
        else
            effects[i] = effects[i-1]
        end
    end
    return effects
end


function changing_scale_pattern_advanced(start, length, scale, offset, extra) -- offset is how much to increase the scale by, extra is how often to
    extra = round(extra)
    if extra == 0 then
        extra = 1
    end
    local effects = {}
    effects[1] = start
    for i = 2, length do
        if ((i-1) % extra == 0) then
            scale = scale + offset
        end
        effects[i] = scale + effects[i-1]
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end


function exponential_pattern_advanced(start, length, scale, offset, extra) --  (scale)^(extra*i), increasing every extra variable
    local effects = {}
    for i = 1, length do
        effects[i] = start * scale^((i-1)*offset)
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end


function polynomial_pattern_advanced(start, length, scale, offset, extra)
    local effects = {}
    for i = 1, length do
        effects[i] = start * (i+ offset) ^ (scale)
        effects[i] = bonus_cap(round(effects[i], -4))
    end
    return effects
end