require("__module-config__/functions.lua")

-- Defining the modules table
-- Defining the modules variable

local modules = {
    speed = {
        name = "speed",
        local_name = "Speed Module ",
        local_description = "Increases machine speed at a cost of increased energy consumption.",
        max_level = settings.startup["max-speed"].value,
        highest_tier = 3,
        scaling = {
            consumption = settings.startup["speed-consumption-scaling"].value,
            speed = settings.startup["speed-speed-scaling"].value,
            productivity = settings.startup["speed-productivity-scaling"].value,
            pollution = settings.startup["speed-pollution-scaling"].value
        },
        graphics = {
            primary_tint = {112/255, 182/255, 255/255},             -- r, g, b, a
            secondary_tint = {99/255, 248/255, 255/255, 1.00}       -- r, g, b, a
        },
        sa_special = "tungsten-carbide",
        sa_special_amount = 1,
        order = "a",
        subgroup = "module",
        override = {
            consumption = manual_override_pattern(settings.startup["speed-consumption-override"].value),
            speed = manual_override_pattern(settings.startup["speed-speed-override"].value), 
            productivity = manual_override_pattern(settings.startup["speed-productivity-override"].value), 
            pollution = manual_override_pattern(settings.startup["speed-pollution-override"].value)
        }
    },
    efficiency = {
        name = "efficiency",
        local_name = "Efficiency Module ",
        local_description = "Decreases machine energy consumption. Minimum energy consumption is 20%.",
        max_level = settings.startup["max-efficiency"].value,
        highest_tier = 3,
        scaling = {
            consumption = settings.startup["efficiency-consumption-scaling"].value,
            speed = settings.startup["efficiency-speed-scaling"].value,
            productivity = settings.startup["efficiency-productivity-scaling"].value,
            pollution = settings.startup["efficiency-pollution-scaling"].value
        },
        graphics = {
            primary_tint = {0/255, 255/255, 0/255},                 -- r, g, b, a
            secondary_tint = {248/255, 255/255, 141/255, 1.00}      -- r, g, b, a
        },
        sa_special = "spoilage",
        sa_special_amount = 5,
        order = "b",
        subgroup = "module",
        override = {
            consumption = manual_override_pattern(settings.startup["efficiency-consumption-override"].value), 
            speed = manual_override_pattern(settings.startup["efficiency-speed-override"].value), 
            productivity = manual_override_pattern(settings.startup["efficiency-productivity-override"].value), 
            pollution = manual_override_pattern(settings.startup["efficiency-pollution-override"].value)
        }
    },
    productivity = {
        name = "productivity",
        local_name = "Productivity Module ",
        local_description = "Machine will create extra products at a cost of increased energy consumption and reduced speed. \n Usable only on intermediate products.",
        max_level = settings.startup["max-productivity"].value,
        highest_tier = 3,
        scaling = {
            consumption = settings.startup["productivity-consumption-scaling"].value,
            speed = settings.startup["productivity-speed-scaling"].value,
            productivity = settings.startup["productivity-productivity-scaling"].value,
            pollution = settings.startup["productivity-pollution-scaling"].value
        },
        graphics = {
            primary_tint = {209/255, 98/255, 66/255},               -- r, g, b, a
            secondary_tint = {255/255, 255/255, 78/255, 1.00}       -- r, g, b, a
        },
        sa_special = "biter-egg",
        sa_special_amount = 1,
        order = "c",
        subgroup = "module",
        override = {
            consumption = manual_override_pattern(settings.startup["productivity-consumption-override"].value), 
            speed = manual_override_pattern(settings.startup["productivity-speed-override"].value), 
            productivity = manual_override_pattern(settings.startup["productivity-productivity-override"].value), 
            pollution = manual_override_pattern(settings.startup["productivity-pollution-override"].value)
        }
    }
}

-- Optional Quality Compatability


if mods["quality"] then
    for _, module_type in pairs({"speed", "productivity", "efficiency"}) do
        modules[module_type].scaling.quality = settings.startup[module_type.."-quality-scaling"].value
        modules[module_type].override.quality = manual_override_pattern(settings.startup[module_type.."-quality-override"].value)
    end
    modules.quality = {
        name = "quality",
        local_name = "Quality Module ",
        local_description = "Allows machines to make higher quality products.",
        max_level = settings.startup["max-quality"].value,
        highest_tier = 3,
        scaling = {
            consumption = settings.startup["quality-consumption-scaling"].value,
            speed = settings.startup["quality-speed-scaling"].value,
            productivity = settings.startup["quality-productivity-scaling"].value,
            pollution = settings.startup["quality-pollution-scaling"].value,
            quality = settings.startup["quality-quality-scaling"].value
        },
        graphics = {
            primary_tint = {255/255, 255/255, 255/255},              -- r, g, b, a
            secondary_tint = {220/255, 40/255, 40/255, 1.00}          -- r, g, b, a
        },
        sa_special = "superconductor",
        sa_special_amount = 1,
        order = "d",
        subgroup = "module",
        override = {
            consumption = manual_override_pattern(settings.startup["quality-consumption-override"].value), 
            speed = manual_override_pattern(settings.startup["quality-speed-override"].value), 
            productivity = manual_override_pattern(settings.startup["quality-productivity-override"].value), 
            pollution = manual_override_pattern(settings.startup["quality-pollution-override"].value), 
            quality = manual_override_pattern(settings.startup["quality-quality-override"].value)
        }
    }
end

-- Moving to new items tab
if settings.startup["more-tabs"].value then
    modules.speed.subgroup = "speed-modules"
    modules.efficiency.subgroup = "efficiency-modules"
    modules.productivity.subgroup = "productivity-modules"
    if mods["quality"] then
        modules.quality.subgroup = "quality-modules"
    end
end

-- Generating the effects
modules["speed"]["effect"] = generate_effects(modules, "speed")
modules["efficiency"]["effect"] = generate_effects(modules, "efficiency")
modules["productivity"]["effect"] = generate_effects(modules, "productivity")
if mods["quality"] then
    modules["quality"]["effect"] = generate_effects(modules, "quality")
end

return modules